@@ -20,6 +20,7 @@ GTKDOCIZE=`which gtkdocize`
 if test -z $GTKDOCIZE; then
         echo "*** No gtk-doc support ***"
         echo "EXTRA_DIST =" > gtk-doc.make
+        echo "CLEANFILES =" >> gtk-doc.make
 else
         gtkdocize || exit $?
         sed -e 's#) --mode=compile#) --tag=CC --mode=compile#' gtk-doc.make \
@@ -40,5 +41,5 @@ fi
 
 cd $olddir
 
-$srcdir/configure --disable-static "$@" && \
+$srcdir/configure --disable-static --enable-maintainer-flags "$@" && \
   echo "Now type 'make' to compile $PROJECT."
