@@ -217,7 +217,8 @@ mx_combo_box_unmap (ClutterActor *actor)
 
   CLUTTER_ACTOR_CLASS (mx_combo_box_parent_class)->unmap (actor);
 
-  clutter_actor_unmap (priv->label);
+  if (priv->label)
+    clutter_actor_unmap (priv->label);
 
   if (priv->icon)
     clutter_actor_unmap (priv->icon);
@@ -319,7 +320,6 @@ mx_combo_box_get_preferred_height (ClutterActor *actor,
                                    gfloat       *natural_height_p)
 {
   MxComboBoxPrivate *priv = MX_COMBO_BOX (actor)->priv;
-  gfloat width;
   gfloat min_label_h, nat_label_h;
   gfloat min_icon_h = 0, nat_icon_h = 0;
   gfloat min_marker_h = 0, nat_marker_h = 0;
@@ -328,8 +328,6 @@ mx_combo_box_get_preferred_height (ClutterActor *actor,
 
   mx_widget_get_padding (MX_WIDGET (actor), &padding);
 
-  width = for_width - padding.left - padding.right;
-
   clutter_actor_get_preferred_height (priv->label,
                                       -1,
                                       &min_label_h,
@@ -522,8 +520,7 @@ mx_combo_box_update_menu (MxComboBox *box)
 }
 
 static gboolean
-mx_combo_box_button_press_event (ClutterActor       *actor,
-                                 ClutterButtonEvent *event)
+mx_combo_box_open_menu (MxComboBox *actor)
 {
   ClutterActor *menu;
 
@@ -536,26 +533,21 @@ mx_combo_box_button_press_event (ClutterActor       *actor,
   return TRUE;
 }
 
+static gboolean
+mx_combo_box_button_press_event (ClutterActor       *actor,
+                                 ClutterButtonEvent *event)
+{
+  return mx_combo_box_open_menu (MX_COMBO_BOX (actor));
+}
+
 static gboolean
 mx_combo_box_key_press_event (ClutterActor    *actor,
                               ClutterKeyEvent *event)
 {
-  MxComboBoxPrivate *priv = MX_COMBO_BOX (actor)->priv;
-  gint n_items;
-
-  n_items = g_slist_length (priv->actions);
-
   switch (event->keyval)
     {
-    case CLUTTER_KEY_Down:
-      mx_combo_box_set_index (MX_COMBO_BOX (actor),
-                              CLAMP (priv->index + 1, 0, n_items -1));
-      return TRUE;
-
-    case CLUTTER_KEY_Up:
-      mx_combo_box_set_index (MX_COMBO_BOX (actor),
-                              CLAMP (priv->index - 1, 0, n_items -1));
-      return TRUE;
+    case CLUTTER_KEY_Return:
+      return mx_combo_box_open_menu (MX_COMBO_BOX (actor));
 
     default:
       return FALSE;
@@ -858,6 +850,33 @@ mx_combo_box_remove_text (MxComboBox *box,
   mx_combo_box_update_menu (box);
 }
 
+/**
+ * mx_combo_box_remove_all:
+ * @box: A #MxComboBox
+ *
+ * Remove all the items of @box
+ *
+ * Since: 1.4
+ */
+void
+mx_combo_box_remove_all (MxComboBox *box)
+{
+  MxComboBoxPrivate *priv = box->priv;
+  GSList *l;
+
+  g_return_if_fail (MX_IS_COMBO_BOX (box));
+
+  l = priv->actions;
+  while (l)
+    {
+      g_object_unref (l->data);
+      l = g_slist_delete_link (l, l);
+    }
+  priv->actions = NULL;
+
+  mx_combo_box_update_menu (box);
+}
+
 /**
  * mx_combo_box_set_active_text:
  * @box: A #MxComboBox
@@ -998,7 +1017,7 @@ mx_combo_box_set_index (MxComboBox *box,
   if (!item)
     {
       box->priv->index = -1;
-      clutter_text_set_text ((ClutterText*) box->priv->label, NULL);
+      clutter_text_set_text ((ClutterText*) box->priv->label, "");
       return;
     }
 
@@ -1038,9 +1057,9 @@ mx_combo_box_set_index (MxComboBox *box,
  *
  * Get the index of the last item selected
  *
- * Returns: const gint
+ * Returns: gint
  */
-const gint
+gint
 mx_combo_box_get_index (MxComboBox *box)
 {
   g_return_val_if_fail (MX_IS_COMBO_BOX (box), 0);
