@@ -24,6 +24,8 @@
 #include <unistd.h>
 #include <fcntl.h>
 
+#include "mx-private.h"
+
 struct _MxStyleSheet
 {
   GList *selectors;
@@ -42,6 +44,8 @@ struct _MxSelector
   MxSelector *ancestor;
   GHashTable *style;
   const gchar *filename; /* origin of this selector */
+  guint line;
+  guint position;
   gint priority;
 };
 
@@ -159,11 +163,11 @@ css_parse_key_value (GScanner *scanner, gchar **key, gchar **value)
           return ';';
         }
 
-      token = g_scanner_peek_next_token (scanner);
+      g_scanner_peek_next_token (scanner);
     }
 
   /* semi colon */
-  token = g_scanner_get_next_token (scanner);
+  g_scanner_get_next_token (scanner);
   if (scanner->value.v_char != ';')
     return ';';
 
@@ -219,17 +223,18 @@ css_parse_simple_selector (GScanner      *scanner,
                            MxSelector    *selector)
 {
   guint token;
+  gchar *tmp;
 
   /* parse optional type (either '*' or an identifier) */
   token = g_scanner_peek_next_token (scanner);
   switch (token)
     {
     case '*':
-      token = g_scanner_get_next_token (scanner);
+      g_scanner_get_next_token (scanner);
       selector->type = g_strdup ("*");
       break;
     case G_TOKEN_IDENTIFIER:
-      token = g_scanner_get_next_token (scanner);
+      g_scanner_get_next_token (scanner);
       selector->type = g_strdup (scanner->value.v_identifier);
       break;
     default:
@@ -244,7 +249,7 @@ css_parse_simple_selector (GScanner      *scanner,
         {
           /* id */
         case '#':
-          token = g_scanner_get_next_token (scanner);
+          g_scanner_get_next_token (scanner);
           token = g_scanner_get_next_token (scanner);
           if (token != G_TOKEN_IDENTIFIER)
             return G_TOKEN_IDENTIFIER;
@@ -252,7 +257,7 @@ css_parse_simple_selector (GScanner      *scanner,
           break;
           /* class */
         case '.':
-          token = g_scanner_get_next_token (scanner);
+          g_scanner_get_next_token (scanner);
           token = g_scanner_get_next_token (scanner);
           if (token != G_TOKEN_IDENTIFIER)
             return G_TOKEN_IDENTIFIER;
@@ -260,11 +265,22 @@ css_parse_simple_selector (GScanner      *scanner,
           break;
           /* pseudo-class */
         case ':':
-          token = g_scanner_get_next_token (scanner);
+          g_scanner_get_next_token (scanner);
           token = g_scanner_get_next_token (scanner);
           if (token != G_TOKEN_IDENTIFIER)
             return G_TOKEN_IDENTIFIER;
-          selector->pseudo_class = g_strdup (scanner->value.v_identifier);
+
+          tmp = selector->pseudo_class;
+
+          if (selector->pseudo_class)
+            selector->pseudo_class = g_strconcat (selector->pseudo_class, ":",
+                                                  scanner->value.v_identifier,
+                                                  NULL);
+          else
+            selector->pseudo_class = g_strdup (scanner->value.v_identifier);
+
+          g_free (tmp);
+
           break;
 
           /* unhandled */
@@ -277,45 +293,77 @@ css_parse_simple_selector (GScanner      *scanner,
   return G_TOKEN_NONE;
 }
 
-#ifdef MX_DEBUG_CSS
-static void
-print_selector (MxSelector *selector, gint indent)
+static char*
+selector_to_string (MxSelector *selector)
 {
-  int i;
+  gchar *ancestor, *string, *tmp, *parent, *ret;
 
-  for (i = 0; i < indent; i++)
-      printf ("  ");
+  if (!selector)
+    return NULL;
 
-  printf ("%s.%s#%s:%s\n", selector->type, selector->class, selector->id,
-          selector->pseudo_class);
+  tmp = selector_to_string (selector->ancestor);
+  if (tmp)
+    ancestor = g_strconcat (tmp, " ", NULL);
+  else
+    ancestor = NULL;
+  g_free (tmp);
 
-  if (selector->parent)
-    {
-      for (i = 0; i < indent; i++)
-        printf ("  ");
-      printf ("p");
-      print_selector (selector->parent, indent + 1);
-    }
+  tmp = selector_to_string (selector->parent);
+  if (tmp)
+    parent = g_strconcat (tmp, " > ", NULL);
+  else
+    parent = NULL;
+  g_free (tmp);
 
-  if (selector->ancestor)
-    {
-      for (i = 0; i < indent; i++)
-        printf ("  ");
-      printf ("a");
-      print_selector (selector->ancestor, indent + 1);
-    }
+  string = g_strdup_printf ("%s%s%s%s%s%s%s",
+                            (selector->type) ?  selector->type : "",
+                            (selector->class) ? "." : "",
+                            (selector->class) ? selector->class : "",
+                            (selector->id) ? "#" : "",
+                            (selector->id) ? selector->id : "",
+                            (selector->pseudo_class) ? ":" : "",
+                            (selector->pseudo_class)
+                            ? selector->pseudo_class : "");
+
+  ret = g_strconcat ((ancestor) ? ancestor : "",
+                     (parent) ? parent : "",
+                     string, NULL);
+
+  g_free (string);
+
+  g_free (ancestor);
+  g_free (parent);
+
+  return ret;
+}
+
+static void
+print_selector (MxSelector *selector,
+                gint        score)
+{
+  gchar *string;
+
+  string = selector_to_string (selector);
+
+  MX_NOTE (CSS, "%5d: %s", score, string);
+
+  g_free (string);
 }
-#endif
 
 
 static MxSelector *
-mx_selector_new (const gchar *filename, gint priority)
+mx_selector_new (const gchar *filename,
+                 gint         priority,
+                 guint        line,
+                 guint        position)
 {
   MxSelector *s;
 
   s = g_slice_new0 (MxSelector);
   s->filename = filename;
   s->priority = priority;
+  s->position = position;
+  s->line = line;
 
   return s;
 }
@@ -366,7 +414,8 @@ css_parse_ruleset (GScanner *scanner, GList **selectors)
            * should use the previous selector to match an ancestor */
 
           selector = mx_selector_new (scanner->input_name,
-                                      GPOINTER_TO_INT (scanner->user_data));
+                                      GPOINTER_TO_INT (scanner->user_data),
+                                      scanner->line, scanner->position);
           *selectors = g_list_prepend (*selectors, selector);
 
           if (parent)
@@ -391,7 +440,8 @@ css_parse_ruleset (GScanner *scanner, GList **selectors)
           parent = selector;
 
           selector = mx_selector_new (scanner->input_name,
-                                      GPOINTER_TO_INT (scanner->user_data));
+                                      GPOINTER_TO_INT (scanner->user_data),
+                                      scanner->line, scanner->position);
           *selectors = g_list_prepend (*selectors, selector);
 
           /* remove parent from list of selectors and link it to the new
@@ -409,7 +459,8 @@ css_parse_ruleset (GScanner *scanner, GList **selectors)
           g_scanner_get_next_token (scanner);
 
           selector = mx_selector_new (scanner->input_name,
-                                      GPOINTER_TO_INT (scanner->user_data));
+                                      GPOINTER_TO_INT (scanner->user_data),
+                                      scanner->line, scanner->position);
           *selectors = g_list_prepend (*selectors, selector);
 
           token = css_parse_simple_selector (scanner, selector);
@@ -521,6 +572,45 @@ css_parse_file (MxStyleSheet *sheet,
     return FALSE;
 }
 
+static gboolean
+list_contains (const gchar *needle,
+               gint         needle_len,
+               const gchar *haystack,
+               gchar        seperator)
+{
+  const gchar *start;
+
+  for (start = haystack; start; start = strchr (start, seperator))
+    {
+      gint len;
+      gchar *next_seperator;
+
+      /* move to the character after the separator */
+      if (start[0] == seperator)
+        start = start + 1;
+
+      /* find the end of this haystack item */
+      next_seperator = strchr (start, seperator);
+      if (!next_seperator)
+        len = strlen (start);
+      else
+        len = next_seperator - start;
+
+      /* if the item in the haystack is not the same length as the separator,
+       * then it is not a match */
+      if (len != needle_len)
+        continue;
+
+      if (!strncmp (needle, start, needle_len))
+        return TRUE;
+      else
+        continue;
+    }
+
+  /* needle not found */
+  return FALSE;
+}
+
 static gint
 css_node_matches_selector (MxSelector *selector,
                            MxStylable *stylable)
@@ -528,12 +618,12 @@ css_node_matches_selector (MxSelector *selector,
   gint score;
   gint a, b, c;
 
+  const gchar *type = "*";
   const gchar *class, *pseudo_class;
   const gchar *id;
   ClutterActor *actor;
   MxStylable *parent;
 
-  score = 0;
   a = 0;
   b = 0;
   c = 0;
@@ -550,7 +640,6 @@ css_node_matches_selector (MxSelector *selector,
     }
   else
     {
-      const gchar *type;
       GType type_id;
       gint matched;
       gint depth;
@@ -591,14 +680,49 @@ css_node_matches_selector (MxSelector *selector,
         a += 10;
     }
 
-  /* check psuedo_class */
+  /* check pseudo_class */
   if (selector->pseudo_class)
     {
-      if (!pseudo_class
-          || strcmp (selector->pseudo_class, pseudo_class))
+      gchar *needle;
+      gint n_matches;
+
+      /* if no pseudo class is supplied on the node, return instantly */
+      if (!pseudo_class)
         return -1;
-      else
-        b += 10;
+
+      /* check that each pseudo-class from the selector appears in the
+       * pseudo-classes from the node, i.e. the selector pseudo-class list
+       * is a subset of the node's pseudo-class list */
+      n_matches = 0;
+      for (needle = selector->pseudo_class;
+           needle; needle = strchr (needle, ':'))
+        {
+          gint needle_len;
+          gchar *next;
+
+          /* move beyond ':' */
+          if (needle[0] == ':')
+            needle++;
+
+          /* calculate the length of this needle */
+          next = strchr (needle, ':');
+          if (next)
+            needle_len = next - needle;
+          else
+            needle_len = strlen (needle);
+
+          /* if the pseudo-class from the selector does not appear in the
+           * list of pseudo-classes from the node, then this is not a
+           * match */
+          if (!list_contains (needle, needle_len, pseudo_class, ':'))
+            return -1;
+          else
+            n_matches++;
+        }
+
+      /* increase the 'b' score by the number of pseudo-classes in the
+       * selector */
+      b = b + (10 * n_matches);
     }
 
   /* check class */
@@ -688,8 +812,21 @@ static gint
 compare_selector_matches (SelectorMatch *a,
                           SelectorMatch *b)
 {
-  gint priority = a->selector->priority - b->selector->priority;
-  return priority ? priority : (a->score - b->score);
+  gint priority;
+  guint line;
+  guint position;
+  gint score;
+
+  if ((score = a->score - b->score) != 0)
+    return score;
+  else if ((priority = a->selector->priority - b->selector->priority) != 0)
+    return priority;
+  else if ((line = a->selector->line - b->selector->line) != 0)
+    return line;
+  else if ((position = a->selector->position - b->selector->position) != 0)
+    return position;
+  else
+    return 0;
 }
 
 struct _css_table_copy_data
@@ -722,14 +859,32 @@ GHashTable *
 mx_style_sheet_get_properties (MxStyleSheet *sheet,
                                MxStylable   *node)
 {
+  GTimer *timer = NULL;
   GList *l, *matching_selectors = NULL;
   SelectorMatch *selector_match = NULL;
   GHashTable *result;
 
+  if (_mx_debug (MX_DEBUG_CSS))
+    {
+      const char *id = clutter_actor_get_name (CLUTTER_ACTOR (node));
+      const char *class = mx_stylable_get_style_class (node);
+      const char *pseudo_class = mx_stylable_get_style_pseudo_class (node);
+      const char *type_name = G_OBJECT_TYPE_NAME (node);
+
+      timer = g_timer_new ();
+      g_print ("\x1b[1m");
+      MX_NOTE (CSS, "Matches for: %s%s%s%s%s%s%s",
+               (type_name) ? type_name : "",
+               (class) ? "." : "",
+               (class) ? class : "",
+               (id) ? "#" : "",
+               (id) ? id : "",
+               (pseudo_class) ? ":" : "",
+               (pseudo_class) ? pseudo_class : "");
+      g_print ("\x1b[22m");
+    }
+
   /* find matching selectors */
-#ifdef MX_DEBUG_CSS
-  GTimer *timer = g_timer_new ();
-#endif
   for (l = sheet->selectors; l; l = l->next)
     {
       gint score;
@@ -766,27 +921,21 @@ mx_style_sheet_get_properties (MxStyleSheet *sheet,
       g_hash_table_foreach (match->selector->style, (GHFunc) css_table_copy,
                             &copy_data);
 
-#ifdef MX_DEBUG_CSS
-          printf ("(%6d) ", match->score);
-          print_selector (match->selector, 3);
-#endif
+      if (_mx_debug (MX_DEBUG_CSS))
+        print_selector (match->selector, match->score);
     }
 
   g_list_foreach (matching_selectors, (GFunc) free_selector_match, NULL);
   g_list_free (matching_selectors);
 
-#ifdef MX_DEBUG_CSS
+  if (_mx_debug (MX_DEBUG_CSS))
+    {
+      g_print ("\x1b[2m");
+      MX_NOTE (CSS, "%fs", g_timer_elapsed (timer, NULL));
+      g_print ("\x1b[0m");
+      g_timer_destroy (timer);
+    }
 
-  g_timer_stop (timer);
-  static int count = 1;
-  static double avg = 0, new;
-  new = g_timer_elapsed (timer, NULL);
-  avg = ((avg * count + new) / (count + 1));
-  count++;
-  printf ("%f - finished (total: %d, average: %f) %s \n",
-          new, count, avg, G_OBJECT_CLASS_NAME (G_OBJECT_GET_CLASS (node)));
-  printf ("----\n");
-#endif
   return result;
 }
 
@@ -827,10 +976,5 @@ mx_style_sheet_add_from_file (MxStyleSheet *sheet,
   result = css_parse_file (sheet, input_name, g_list_length (sheet->filenames));
   sheet->filenames = g_list_prepend (sheet->filenames, input_name);
 
-#ifdef MX_DEBUG_CSS
-  g_list_foreach (sheet->selectors, (GFunc)print_selector, GINT_TO_POINTER (0));
-  printf ("-------\n");
-#endif
-
   return result;
 }
