@@ -178,8 +178,21 @@ static void
 mx_progress_bar_unmap (ClutterActor *actor)
 {
   MxProgressBarPrivate *priv = MX_PROGRESS_BAR (actor)->priv;
+
+  if (priv->fill)
+    clutter_actor_unmap (priv->fill);
+
   CLUTTER_ACTOR_CLASS (mx_progress_bar_parent_class)->unmap (actor);
-  clutter_actor_unmap (priv->fill);
+}
+
+static void
+mx_progress_bar_apply_style (MxWidget *widget,
+                             MxStyle  *style)
+{
+  MxProgressBarPrivate *priv = MX_PROGRESS_BAR (widget)->priv;
+
+  if (priv->fill != NULL)
+    mx_stylable_set_style (MX_STYLABLE (priv->fill), style);
 }
 
 static void
@@ -241,6 +254,7 @@ mx_progress_bar_class_init (MxProgressBarClass *klass)
   GParamSpec *pspec;
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
   ClutterActorClass *actor_class = CLUTTER_ACTOR_CLASS (klass);
+  MxWidgetClass *widget_class = MX_WIDGET_CLASS (klass);
 
   g_type_class_add_private (klass, sizeof (MxProgressBarPrivate));
 
@@ -256,6 +270,8 @@ mx_progress_bar_class_init (MxProgressBarClass *klass)
   actor_class->map = mx_progress_bar_map;
   actor_class->unmap = mx_progress_bar_unmap;
 
+  widget_class->apply_style = mx_progress_bar_apply_style;
+
   pspec = g_param_spec_double ("progress",
                                "Progress",
                                "Progress",
