@@ -250,9 +250,14 @@ mx_scroll_bar_unmap (ClutterActor *actor)
 
   CLUTTER_ACTOR_CLASS (mx_scroll_bar_parent_class)->unmap (actor);
 
-  clutter_actor_unmap (priv->bw_stepper);
-  clutter_actor_unmap (priv->fw_stepper);
-  clutter_actor_unmap (priv->trough);
+  if (priv->bw_stepper)
+    clutter_actor_unmap (priv->bw_stepper);
+
+  if (priv->fw_stepper)
+    clutter_actor_unmap (priv->fw_stepper);
+
+  if (priv->trough)
+    clutter_actor_unmap (priv->trough);
 
   if (priv->handle)
     clutter_actor_unmap (priv->handle);
@@ -556,13 +561,11 @@ mx_scroll_bar_constructor (GType                  type,
   GObjectClass *gobject_class;
   GObject *obj;
   MxScrollBar *bar;
-  MxScrollBarPrivate *priv;
 
   gobject_class = G_OBJECT_CLASS (mx_scroll_bar_parent_class);
   obj = gobject_class->constructor (type, n_properties, properties);
 
   bar  = MX_SCROLL_BAR (obj);
-  priv = MX_SCROLL_BAR_GET_PRIVATE (bar);
 
   g_signal_connect (bar, "notify::reactive",
                     G_CALLBACK (bar_reactive_notify_cb), NULL);
@@ -1119,7 +1122,7 @@ mx_scroll_bar_init (MxScrollBar *self)
   self->priv->handle = (ClutterActor *) mx_button_new ();
   mx_stylable_set_style_class (MX_STYLABLE (self->priv->handle), "hhandle");
   clutter_actor_set_parent (CLUTTER_ACTOR (self->priv->handle),
-                            self->priv->trough);
+                            CLUTTER_ACTOR (self));
   g_signal_connect (self->priv->handle, "button-press-event",
                     G_CALLBACK (handle_button_press_event_cb), self);
 
