@@ -29,6 +29,8 @@
  * allocated its preferred size or larger, if the fill option is set. If
  * the fill option isn't set, a child's position will be determined by its
  * alignment properties.
+ *
+ * Since: 1.2
  */
 
 #include "mx-stack.h"
@@ -36,6 +38,8 @@
 #include "mx-focusable.h"
 #include "mx-utils.h"
 
+#include <string.h>
+
 static void clutter_container_iface_init (ClutterContainerIface *iface);
 static void mx_focusable_iface_init (MxFocusableIface *iface);
 
@@ -52,6 +56,8 @@ struct _MxStackPrivate
 {
   GList        *children;
   ClutterActor *current_focus;
+
+  ClutterActorBox allocation;
 };
 
 /* ClutterContainerIface */
@@ -368,6 +374,8 @@ mx_stack_get_preferred_width (ClutterActor *actor,
   for (c = priv->children; c; c = c->next)
     {
       ClutterActor *child = c->data;
+      if (!CLUTTER_ACTOR_IS_VISIBLE (child))
+        continue;
       clutter_actor_get_preferred_width (child, for_height,
                                          &child_min_width, &child_nat_width);
       if (child_min_width > min_width)
@@ -405,6 +413,8 @@ mx_stack_get_preferred_height (ClutterActor *actor,
   for (c = priv->children; c; c = c->next)
     {
       ClutterActor *child = c->data;
+      if (!CLUTTER_ACTOR_IS_VISIBLE (child))
+        continue;
       clutter_actor_get_preferred_height (child, for_width,
                                           &child_min_height, &child_nat_height);
       if (child_min_height > min_height)
@@ -436,22 +446,139 @@ mx_stack_allocate (ClutterActor           *actor,
 
   mx_widget_get_available_area (MX_WIDGET (actor), box, &avail_space);
 
+  memcpy (&priv->allocation, box, sizeof (priv->allocation));
+
   for (c = priv->children; c; c = c->next)
     {
-      gboolean x_fill, y_fill;
+      gboolean x_fill, y_fill, fit, crop;
       MxAlign x_align, y_align;
 
       ClutterActor *child = c->data;
       ClutterActorBox child_box = avail_space;
 
+      if (!CLUTTER_ACTOR_IS_VISIBLE (child))
+        continue;
+
       clutter_container_child_get (CLUTTER_CONTAINER (actor),
                                    child,
                                    "x-fill", &x_fill,
                                    "y-fill", &y_fill,
                                    "x-align", &x_align,
                                    "y-align", &y_align,
+                                   "fit", &fit,
+                                   "crop", &crop,
                                    NULL);
 
+      /* when "crop" is set, fit and fill properties are ignored */
+      if (crop)
+        {
+          gfloat available_height, available_width;
+          gfloat natural_width, natural_height;
+          gfloat ratio_width, ratio_height, ratio_child;
+
+          available_width  = avail_space.x2 - avail_space.x1;
+          available_height = avail_space.y2 - avail_space.y1;
+
+          clutter_actor_get_preferred_size (child,
+                                            NULL, NULL,
+                                            &natural_width,
+                                            &natural_height);
+
+          ratio_child = natural_width / natural_height;
+          ratio_width = available_width / natural_width;
+          ratio_height = available_height / natural_height;
+          if (ratio_width > ratio_height)
+            {
+              natural_width = available_width;
+              natural_height = natural_width / ratio_child;
+            }
+          else
+            {
+              natural_height = available_height;
+              natural_width = ratio_child * natural_height;
+            }
+
+          child_box.x1 = (available_width - natural_width) / 2;
+          child_box.y1 = (available_height - natural_height) / 2;
+          child_box.x2 = natural_width;
+          child_box.y2 = natural_height;
+
+          clutter_actor_allocate (child, &child_box, flags);
+          continue;
+        }
+
+      /* when "fit" is set, fill properties are ignored */
+      if (fit)
+        {
+          gfloat available_height, available_width, width, height;
+          gfloat min_width, natural_width, min_height, natural_height;
+          ClutterRequestMode request_mode;
+
+          available_height = avail_space.y2 - avail_space.y1;
+          available_width  = avail_space.x2 - avail_space.x1;
+          request_mode = clutter_actor_get_request_mode (child);
+
+          if (request_mode == CLUTTER_REQUEST_HEIGHT_FOR_WIDTH)
+            {
+              clutter_actor_get_preferred_width (child, available_height,
+                                                 &min_width,
+                                                 &natural_width);
+              width = CLAMP (natural_width, min_width, available_width);
+
+              clutter_actor_get_preferred_height (child, width,
+                                                  &min_height,
+                                                  &natural_height);
+              height = CLAMP (natural_height, min_height, available_height);
+            }
+          else
+            {
+              clutter_actor_get_preferred_height (child, available_width,
+                                                  &min_height,
+                                                  &natural_height);
+              height = CLAMP (natural_height, min_height, available_height);
+
+              clutter_actor_get_preferred_width (child, height,
+                                                 &min_width,
+                                                 &natural_width);
+              width = CLAMP (natural_width, min_width, available_width);
+            }
+
+          child_box.x1 = 0;
+          child_box.y1 = 0;
+
+          switch (x_align)
+            {
+            case MX_ALIGN_START:
+              break;
+            case MX_ALIGN_MIDDLE:
+              child_box.x1 += (gint)(available_width / 2 - width / 2);
+              break;
+
+            case MX_ALIGN_END:
+              child_box.x1 = avail_space.x2 - width;
+              break;
+            }
+
+          switch (y_align)
+            {
+            case MX_ALIGN_START:
+              break;
+            case MX_ALIGN_MIDDLE:
+              child_box.y1 += (gint)(available_height / 2 - height / 2);
+              break;
+
+            case MX_ALIGN_END:
+              child_box.y1 = avail_space.y2 - height;
+              break;
+            }
+
+          child_box.x2 = child_box.x1 + width;
+          child_box.y2 = child_box.y1 + height;
+
+          clutter_actor_allocate (child, &child_box, flags);
+          continue;
+        }
+
       /* Adjust the available space when not filling, otherwise
        * actors that support width-for-height or height-for-width
        * allocation won't shrink correctly.
@@ -522,21 +649,57 @@ mx_stack_allocate (ClutterActor           *actor,
 }
 
 static void
-mx_stack_paint (ClutterActor *actor)
+mx_stack_paint_children (ClutterActor *actor)
 {
-  GList *c;
-
   MxStackPrivate *priv = MX_STACK (actor)->priv;
 
+  GList *c;
+
   for (c = priv->children; c; c = c->next)
-    clutter_actor_paint (c->data);
+    {
+      ClutterActor *child = c->data;
+      gboolean crop;
+
+      if (!CLUTTER_ACTOR_IS_VISIBLE (child))
+        continue;
+
+      clutter_container_child_get (CLUTTER_CONTAINER (actor),
+                                   child,
+                                   "crop", &crop,
+                                   NULL);
+
+      if (crop)
+        {
+          /* clip */
+          cogl_clip_push_rectangle (priv->allocation.x1,
+                                    priv->allocation.y1,
+                                    priv->allocation.x2,
+                                    priv->allocation.y2);
+          clutter_actor_paint (c->data);
+          cogl_clip_pop ();
+        }
+      else
+        clutter_actor_paint (c->data);
+    }
+}
+
+static void
+mx_stack_paint (ClutterActor *actor)
+{
+  /* allow MxWidget to paint the background */
+  CLUTTER_ACTOR_CLASS (mx_stack_parent_class)->paint (actor);
+
+
+  mx_stack_paint_children (actor);
 }
 
 static void
 mx_stack_pick (ClutterActor       *actor,
                const ClutterColor *color)
 {
-  mx_stack_paint (actor);
+  CLUTTER_ACTOR_CLASS (mx_stack_parent_class)->pick (actor, color);
+
+  mx_stack_paint_children (actor);
 }
 
 static void
@@ -568,6 +731,8 @@ mx_stack_init (MxStack *self)
  * Create a new #MxStack.
  *
  * Returns: a newly allocated #MxStack
+ *
+ * Since: 1.2
  */
 ClutterActor *
 mx_stack_new ()
