@@ -1,3 +1,4 @@
+/* -*- mode: C; c-file-style: "gnu"; indent-tabs-mode: nil; -*- */
 /*
  * mx-table-child.h: Table child implementation
  *
@@ -41,9 +42,9 @@
 enum {
   CHILD_PROP_0,
 
-  CHILD_PROP_COL,
+  CHILD_PROP_COLUMN,
   CHILD_PROP_ROW,
-  CHILD_PROP_COL_SPAN,
+  CHILD_PROP_COLUMN_SPAN,
   CHILD_PROP_ROW_SPAN,
   CHILD_PROP_X_EXPAND,
   CHILD_PROP_Y_EXPAND,
@@ -51,7 +52,6 @@ enum {
   CHILD_PROP_Y_ALIGN,
   CHILD_PROP_X_FILL,
   CHILD_PROP_Y_FILL,
-  CHILD_PROP_ALLOCATE_HIDDEN,
 };
 
 G_DEFINE_TYPE (MxTableChild, mx_table_child, CLUTTER_TYPE_CHILD_META);
@@ -67,7 +67,7 @@ table_child_set_property (GObject      *gobject,
 
   switch (prop_id)
     {
-    case CHILD_PROP_COL:
+    case CHILD_PROP_COLUMN:
       child->col = g_value_get_int (value);
       _mx_table_update_row_col (table, -1, child->col);
       clutter_actor_queue_relayout (CLUTTER_ACTOR (table));
@@ -77,7 +77,7 @@ table_child_set_property (GObject      *gobject,
       _mx_table_update_row_col (table, child->row, -1);
       clutter_actor_queue_relayout (CLUTTER_ACTOR (table));
       break;
-    case CHILD_PROP_COL_SPAN:
+    case CHILD_PROP_COLUMN_SPAN:
       child->col_span = g_value_get_int (value);
       clutter_actor_queue_relayout (CLUTTER_ACTOR (table));
       break;
@@ -94,11 +94,33 @@ table_child_set_property (GObject      *gobject,
       clutter_actor_queue_relayout (CLUTTER_ACTOR (table));
       break;
     case CHILD_PROP_X_ALIGN:
-      child->x_align = g_value_get_double (value);
+      switch (g_value_get_enum (value))
+        {
+        case MX_ALIGN_START:
+          child->x_align = 0.0;
+          break;
+        case MX_ALIGN_MIDDLE:
+          child->x_align = 0.5;
+          break;
+        case MX_ALIGN_END:
+          child->x_align = 1.0;
+          break;
+        }
       clutter_actor_queue_relayout (CLUTTER_ACTOR (table));
       break;
     case CHILD_PROP_Y_ALIGN:
-      child->y_align = g_value_get_double (value);
+      switch (g_value_get_enum (value))
+        {
+        case MX_ALIGN_START:
+          child->y_align = 0.0;
+          break;
+        case MX_ALIGN_MIDDLE:
+          child->y_align = 0.5;
+          break;
+        case MX_ALIGN_END:
+          child->y_align = 1.0;
+          break;
+        }
       clutter_actor_queue_relayout (CLUTTER_ACTOR (table));
       break;
     case CHILD_PROP_X_FILL:
@@ -126,13 +148,13 @@ table_child_get_property (GObject    *gobject,
 
   switch (prop_id)
     {
-    case CHILD_PROP_COL:
+    case CHILD_PROP_COLUMN:
       g_value_set_int (value, child->col);
       break;
     case CHILD_PROP_ROW:
       g_value_set_int (value, child->row);
       break;
-    case CHILD_PROP_COL_SPAN:
+    case CHILD_PROP_COLUMN_SPAN:
       g_value_set_int (value, child->col_span);
       break;
     case CHILD_PROP_ROW_SPAN:
@@ -145,10 +167,20 @@ table_child_get_property (GObject    *gobject,
       g_value_set_boolean (value, child->y_expand);
       break;
     case CHILD_PROP_X_ALIGN:
-      g_value_set_double (value, child->x_align);
+      if (child->x_align < 1.0/3.0)
+        g_value_set_enum (value, MX_ALIGN_START);
+      else if (child->x_align > 2.0/3.0)
+        g_value_set_enum (value, MX_ALIGN_END);
+      else
+        g_value_set_enum (value, MX_ALIGN_MIDDLE);
       break;
     case CHILD_PROP_Y_ALIGN:
-      g_value_set_double (value, child->y_align);
+      if (child->y_align < 1.0/3.0)
+        g_value_set_enum (value, MX_ALIGN_START);
+      else if (child->y_align > 2.0/3.0)
+        g_value_set_enum (value, MX_ALIGN_END);
+      else
+        g_value_set_enum (value, MX_ALIGN_MIDDLE);
       break;
     case CHILD_PROP_X_FILL:
       g_value_set_boolean (value, child->x_fill);
@@ -172,14 +204,15 @@ mx_table_child_class_init (MxTableChildClass *klass)
   gobject_class->set_property = table_child_set_property;
   gobject_class->get_property = table_child_get_property;
 
-  pspec = g_param_spec_int ("col",
+
+  pspec = g_param_spec_int ("column",
                             "Column Number",
                             "The column the widget resides in",
                             0, G_MAXINT,
                             0,
                             MX_PARAM_READWRITE);
 
-  g_object_class_install_property (gobject_class, CHILD_PROP_COL, pspec);
+  g_object_class_install_property (gobject_class, CHILD_PROP_COLUMN, pspec);
 
   pspec = g_param_spec_int ("row",
                             "Row Number",
@@ -199,14 +232,16 @@ mx_table_child_class_init (MxTableChildClass *klass)
 
   g_object_class_install_property (gobject_class, CHILD_PROP_ROW_SPAN, pspec);
 
-  pspec = g_param_spec_int ("col-span",
+
+  pspec = g_param_spec_int ("column-span",
                             "Column Span",
                             "The number of columns the widget should span",
                             1, G_MAXINT,
                             1,
                             MX_PARAM_READWRITE);
 
-  g_object_class_install_property (gobject_class, CHILD_PROP_COL_SPAN, pspec);
+  g_object_class_install_property (gobject_class, CHILD_PROP_COLUMN_SPAN,
+                                   pspec);
 
   pspec = g_param_spec_boolean ("x-expand",
                                 "X Expand",
@@ -228,21 +263,21 @@ mx_table_child_class_init (MxTableChildClass *klass)
 
   g_object_class_install_property (gobject_class, CHILD_PROP_Y_EXPAND, pspec);
 
-  pspec = g_param_spec_double ("x-align",
-                               "X Alignment",
-                               "X alignment of the widget within the cell",
-                               0, 1,
-                               0.5,
-                               MX_PARAM_READWRITE);
+  pspec = g_param_spec_enum ("x-align",
+                             "X Alignment",
+                             "X alignment of the widget within the cell",
+                             MX_TYPE_ALIGN,
+                             MX_ALIGN_MIDDLE,
+                             MX_PARAM_READWRITE);
 
   g_object_class_install_property (gobject_class, CHILD_PROP_X_ALIGN, pspec);
 
-  pspec = g_param_spec_double ("y-align",
-                               "Y Alignment",
-                               "Y alignment of the widget within the cell",
-                               0, 1,
-                               0.5,
-                               MX_PARAM_READWRITE);
+  pspec = g_param_spec_enum ("y-align",
+                             "Y Alignment",
+                             "Y alignment of the widget within the cell",
+                             MX_TYPE_ALIGN,
+                             MX_ALIGN_MIDDLE,
+                             MX_PARAM_READWRITE);
 
   g_object_class_install_property (gobject_class, CHILD_PROP_Y_ALIGN, pspec);
 
@@ -265,15 +300,6 @@ mx_table_child_class_init (MxTableChildClass *klass)
                                 MX_PARAM_READWRITE);
 
   g_object_class_install_property (gobject_class, CHILD_PROP_Y_FILL, pspec);
-
-  pspec = g_param_spec_boolean ("allocate-hidden",
-                                "Allocate Hidden",
-                                "Whether the child should be allocate even "
-                                "if it is hidden",
-                                TRUE,
-                                MX_PARAM_READWRITE);
-
-  g_object_class_install_property (gobject_class, CHILD_PROP_ALLOCATE_HIDDEN, pspec);
 }
 
 static void
@@ -305,7 +331,7 @@ get_child_meta (MxTable      *table,
 }
 
 /**
- * mx_table_child_get_col_span:
+ * mx_table_child_get_column_span:
  * @table: an #MxTable
  * @child: a #ClutterActor
  *
@@ -313,9 +339,10 @@ get_child_meta (MxTable      *table,
  *
  * Returns: the column span of the child
  */
+
 gint
-mx_table_child_get_col_span (MxTable      *table,
-                             ClutterActor *child)
+mx_table_child_get_column_span (MxTable      *table,
+                                ClutterActor *child)
 {
   MxTableChild *meta;
 
@@ -328,7 +355,7 @@ mx_table_child_get_col_span (MxTable      *table,
 }
 
 /**
- * mx_table_child_set_col_span:
+ * mx_table_child_set_column_span:
  * @table: An #MxTable
  * @child: An #ClutterActor
  * @span: The number of columns to span
@@ -336,16 +363,17 @@ mx_table_child_get_col_span (MxTable      *table,
  * Set the column span of the child.
  *
  */
+
 void
-mx_table_child_set_col_span (MxTable      *table,
-                             ClutterActor *child,
-                             gint          span)
+mx_table_child_set_column_span (MxTable      *table,
+                                ClutterActor *child,
+                                gint          span)
 {
   MxTableChild *meta;
 
   g_return_if_fail (MX_IS_TABLE (table));
   g_return_if_fail (CLUTTER_IS_ACTOR (child));
-  g_return_if_fail (span > 1);
+  g_return_if_fail (span >= 1);
 
   meta = get_child_meta (table, child);
 
@@ -395,7 +423,7 @@ mx_table_child_set_row_span (MxTable      *table,
 
   g_return_if_fail (MX_IS_TABLE (table));
   g_return_if_fail (CLUTTER_IS_ACTOR (child));
-  g_return_if_fail (span > 1);
+  g_return_if_fail (span >= 1);
 
   meta = get_child_meta (table, child);
 
@@ -741,3 +769,106 @@ mx_table_child_set_y_align (MxTable      *table,
   clutter_actor_queue_relayout (child);
 }
 
+/**
+ * mx_table_child_get_column:
+ * @table: an #MxTable
+ * @child: a #ClutterActor
+ *
+ * Get the column of the child.
+ *
+ * Returns: the column of the child
+ */
+gint
+mx_table_child_get_column (MxTable      *table,
+                        ClutterActor *child)
+{
+  MxTableChild *meta;
+
+  g_return_val_if_fail (MX_IS_TABLE (table), -1);
+  g_return_val_if_fail (CLUTTER_IS_ACTOR (child), -1);
+
+  meta = get_child_meta (table, child);
+
+  return meta->col;
+}
+
+/**
+ * mx_table_child_set_column:
+ * @table: a #MxTable
+ * @child: a #ClutterActor
+ * @col: the column of the child
+ *
+ * Set the column of the child
+ */
+void
+mx_table_child_set_column (MxTable      *table,
+                           ClutterActor *child,
+                           gint          col)
+{
+  MxTableChild *meta;
+
+  g_return_if_fail (MX_IS_TABLE (table));
+  g_return_if_fail (CLUTTER_IS_ACTOR (child));
+
+  meta = get_child_meta (table, child);
+
+  if (meta->col == col)
+    return;
+
+  meta->col = col;
+
+  _mx_table_update_row_col (table, -1, meta->col);
+  clutter_actor_queue_relayout (CLUTTER_ACTOR (table));
+}
+
+/**
+ * mx_table_child_get_row:
+ * @table: an #MxTable
+ * @child: a #ClutterActor
+ *
+ * Get the row of the child.
+ *
+ * Returns: the row of the child
+ */
+gint
+mx_table_child_get_row (MxTable      *table,
+                        ClutterActor *child)
+{
+  MxTableChild *meta;
+
+  g_return_val_if_fail (MX_IS_TABLE (table), -1);
+  g_return_val_if_fail (CLUTTER_IS_ACTOR (child), -1);
+
+  meta = get_child_meta (table, child);
+
+  return meta->row;
+}
+
+/**
+ * mx_table_child_set_row:
+ * @table: a #MxTable
+ * @child: a #ClutterActor
+ * @row: the row of the child
+ *
+ * Set the row of the child
+ */
+void
+mx_table_child_set_row (MxTable      *table,
+                        ClutterActor *child,
+                        gint          row)
+{
+  MxTableChild *meta;
+
+  g_return_if_fail (MX_IS_TABLE (table));
+  g_return_if_fail (CLUTTER_IS_ACTOR (child));
+
+  meta = get_child_meta (table, child);
+
+  if (meta->row == row)
+    return;
+
+  meta->row = row;
+
+  _mx_table_update_row_col (table, meta->row, -1);
+  clutter_actor_queue_relayout (CLUTTER_ACTOR (table));
+}
