@@ -127,6 +127,7 @@ G_DEFINE_TYPE_WITH_CODE (MxTable, mx_table, MX_TYPE_WIDGET,
                          G_IMPLEMENT_INTERFACE (MX_TYPE_FOCUSABLE,
                                                 mx_focusable_iface_init));
 
+
 static ClutterActor*
 mx_table_find_actor_at (MxTable *table,
                         int      row,
@@ -173,6 +174,8 @@ mx_table_move_focus (MxFocusable      *focusable,
   if (!childlink)
     return NULL;
 
+  priv->last_focus = from;
+
   child_actor = CLUTTER_ACTOR (childlink->data);
   child_meta = (MxTableChild *) clutter_container_get_child_meta (CLUTTER_CONTAINER (focusable),
                                                                   child_actor);
@@ -190,10 +193,7 @@ mx_table_move_focus (MxFocusable      *focusable,
                                                    MX_FOCUS_HINT_FIRST);
 
               if (focused)
-                {
-                  priv->last_focus = (MxFocusable *)l->data;
-                  return focused;
-                }
+                return focused;
             }
         }
 
@@ -210,10 +210,7 @@ mx_table_move_focus (MxFocusable      *focusable,
                                                    MX_FOCUS_HINT_LAST);
 
               if (focused)
-                {
-                  priv->last_focus = (MxFocusable *)l->data;
-                  return focused;
-                }
+                return focused;
             }
         }
 
@@ -239,10 +236,7 @@ mx_table_move_focus (MxFocusable      *focusable,
                   focused = mx_focusable_accept_focus (MX_FOCUSABLE (found),
                                                        MX_FOCUS_HINT_FIRST);
                   if (focused)
-                    {
-                      priv->last_focus = (MxFocusable *)found;
-                      break;
-                    }
+                    break;
                 }
 
               child_meta = (MxTableChild *) clutter_container_get_child_meta
@@ -277,10 +271,7 @@ mx_table_move_focus (MxFocusable      *focusable,
                   focused = mx_focusable_accept_focus (MX_FOCUSABLE (found),
                                                        MX_FOCUS_HINT_FIRST);
                   if (focused)
-                    {
-                      priv->last_focus = (MxFocusable *)found;
-                      break;
-                    }
+                    break;
                 }
 
               child_meta = (MxTableChild *) clutter_container_get_child_meta
@@ -314,10 +305,7 @@ mx_table_move_focus (MxFocusable      *focusable,
                   focused = mx_focusable_accept_focus (MX_FOCUSABLE (found),
                                                        MX_FOCUS_HINT_FIRST);
                   if (focused)
-                    {
-                      priv->last_focus = (MxFocusable *)found;
-                      break;
-                    }
+                    break;
                 }
 
               child_meta = (MxTableChild *) clutter_container_get_child_meta
@@ -352,10 +340,7 @@ mx_table_move_focus (MxFocusable      *focusable,
                   focused = mx_focusable_accept_focus (MX_FOCUSABLE (found),
                                                        MX_FOCUS_HINT_FIRST);
                   if (focused)
-                    {
-                      priv->last_focus = (MxFocusable *)found;
-                      break;
-                    }
+                    break;
                 }
 
               child_meta = (MxTableChild *) clutter_container_get_child_meta
@@ -441,9 +426,11 @@ mx_container_add_actor (ClutterContainer *container,
 
   clutter_actor_set_parent (actor, CLUTTER_ACTOR (container));
 
-
   priv->children = g_list_append (priv->children, actor);
 
+  /* default position of the actor is 0, 0 */
+  _mx_table_update_row_col (MX_TABLE (container), 0, 0);
+
   clutter_actor_queue_relayout (CLUTTER_ACTOR (container));
 
   g_signal_emit_by_name (container, "actor-added", actor);
@@ -588,7 +575,7 @@ mx_table_depth_sort_cb (gconstpointer a,
                         gconstpointer b)
 {
   gfloat depth_a = clutter_actor_get_depth ((ClutterActor *)a);
-  gfloat depth_b = clutter_actor_get_depth ((ClutterActor *)a);
+  gfloat depth_b = clutter_actor_get_depth ((ClutterActor *)b);
 
   if (depth_a < depth_b)
     return -1;
@@ -746,7 +733,7 @@ mx_table_calculate_col_widths (MxTable *table,
 
   g_array_set_size (priv->columns, 0);
   g_array_set_size (priv->columns, priv->n_cols);
-  columns = (DimensionData *) priv->columns->data;
+  columns = &g_array_index (priv->columns, DimensionData, 0);
 
 
   /* take off the padding values to calculate the allocatable width */
@@ -790,7 +777,7 @@ mx_table_calculate_col_widths (MxTable *table,
       clutter_actor_get_preferred_width (child, -1, &c_min, &c_pref);
 
       col->min_size = MAX (col->min_size, c_min);
-      col->pref_size = MAX (col->pref_size, c_pref);
+      col->final_size = col->pref_size = MAX (col->pref_size, c_pref);
       col->expand = MAX (col->expand, meta->x_expand);
     }
 
@@ -1038,9 +1025,9 @@ mx_table_calculate_row_heights (MxTable *table,
 
   g_array_set_size (priv->rows, 0);
   g_array_set_size (priv->rows, priv->n_rows);
-  rows = (DimensionData*) priv->rows->data;
+  rows = &g_array_index (priv->rows, DimensionData, 0);
 
-  columns = (DimensionData*) priv->columns->data;
+  columns = &g_array_index (priv->columns, DimensionData, 0);
 
   /* Reset the visible rows */
   priv->visible_rows = 0;
@@ -1079,7 +1066,7 @@ mx_table_calculate_row_heights (MxTable *table,
                                           &c_min, &c_pref);
 
       row->min_size = MAX (row->min_size, c_min);
-      row->pref_size = MAX (row->pref_size, c_pref);
+      row->final_size = row->pref_size = MAX (row->pref_size, c_pref);
       row->expand = MAX (row->expand, meta->y_expand);
     }
 
@@ -1090,7 +1077,6 @@ mx_table_calculate_row_heights (MxTable *table,
     {
       MxTableChild *meta;
       ClutterActor *child;
-      DimensionData *row;
       gfloat c_min, c_pref;
       gfloat min_height, pref_height;
       gint start_row, end_row;
@@ -1107,7 +1093,6 @@ mx_table_calculate_row_heights (MxTable *table,
       if (meta->row_span < 2)
         continue;
 
-      row = &rows[meta->row];
       start_row = meta->row;
       end_row = meta->row + meta->row_span - 1;
 
@@ -1337,7 +1322,7 @@ mx_table_preferred_allocate (ClutterActor          *self,
 {
   GList *list;
   gint row_spacing, col_spacing;
-  gint i, table_width, table_height;
+  gint i;
   MxTable *table;
   MxTablePrivate *priv;
   MxPadding padding;
@@ -1351,18 +1336,10 @@ mx_table_preferred_allocate (ClutterActor          *self,
   col_spacing = (priv->col_spacing);
   row_spacing = (priv->row_spacing);
 
-
-  table_height = (int)(box->y2 - box->y1
-                       - padding.top
-                       - padding.bottom);
-  table_width = (int)(box->x2 - box->x1
-                      - padding.right
-                      - padding.left);
-
   mx_table_calculate_dimensions (table, box->x2 - box->x1, box->y2 - box->y1);
 
-  rows = (DimensionData *) priv->rows->data;
-  columns = (DimensionData *) priv->columns->data;
+  rows = &g_array_index (priv->rows, DimensionData, 0);
+  columns = &g_array_index (priv->columns, DimensionData, 0);
 
   for (list = priv->children; list; list = g_list_next (list))
     {
@@ -1519,7 +1496,7 @@ mx_table_get_preferred_width (ClutterActor *self,
 
   mx_table_calculate_dimensions (MX_TABLE (self), -1, for_height);
 
-  columns = (DimensionData*) priv->columns->data;
+  columns = &g_array_index (priv->columns, DimensionData, 0);
 
   total_min_width = padding.left + padding.right
                     + (priv->visible_cols - 1) * (float) priv->col_spacing;
@@ -1558,7 +1535,7 @@ mx_table_get_preferred_height (ClutterActor *self,
   /* use min_widths to help allocation of height-for-width widgets */
   mx_table_calculate_dimensions (MX_TABLE (self), for_width, -1);
 
-  rows = (DimensionData*) priv->rows->data;
+  rows = &g_array_index (priv->rows, DimensionData, 0);
 
   mx_widget_get_padding (MX_WIDGET (self), &padding);
 
@@ -1603,8 +1580,8 @@ mx_table_paint (ClutterActor *self)
       gfloat pos = 0;
       DimensionData *rows, *cols;
 
-      rows = (DimensionData*) priv->rows->data;
-      cols = (DimensionData*) priv->columns->data;
+      rows = &g_array_index (priv->rows, DimensionData, 0);
+      cols = &g_array_index (priv->columns, DimensionData, 0);
 
       clutter_actor_get_size (self, &width, &height);
 
