@@ -257,7 +257,8 @@ mx_toggle_unmap (ClutterActor *actor)
 {
   MxTogglePrivate *priv = MX_TOGGLE (actor)->priv;
 
-  clutter_actor_unmap (priv->handle);
+  if (priv->handle)
+    clutter_actor_unmap (priv->handle);
 
   CLUTTER_ACTOR_CLASS (mx_toggle_parent_class)->unmap (actor);
 }
@@ -328,7 +329,7 @@ mx_toggle_handle_button_press_event (ClutterActor       *actor,
                                      ClutterButtonEvent *event,
                                      MxToggle           *toggle)
 {
-  if (mx_widget_get_disabled (MX_WIDGET (actor)))
+  if (mx_widget_get_disabled (MX_WIDGET (toggle)))
     return FALSE;
 
   clutter_grab_pointer (actor);
@@ -347,7 +348,9 @@ mx_toggle_handle_button_release_event (ClutterActor       *actor,
                                        ClutterButtonEvent *event,
                                        MxToggle           *toggle)
 {
-  if (mx_widget_get_disabled (MX_WIDGET (actor)))
+  ClutterActorBox box;
+
+  if (mx_widget_get_disabled (MX_WIDGET (toggle)))
     return FALSE;
 
   if (toggle->priv->last_move == 0)
@@ -361,6 +364,12 @@ mx_toggle_handle_button_release_event (ClutterActor       *actor,
 
   clutter_ungrab_pointer ();
 
+  /* ensure the hover state is removed if the pointer left the handle
+   * during the grab */
+  clutter_actor_get_allocation_box (actor, &box);
+  if (!clutter_actor_box_contains (&box, event->x, event->y))
+    mx_stylable_style_pseudo_class_remove (MX_STYLABLE (actor), "hover");
+
   return TRUE;
 }
 
@@ -371,7 +380,7 @@ mx_toggle_handle_motion_event (ClutterActor       *actor,
 {
   MxTogglePrivate *priv = toggle->priv;
 
-  if (mx_widget_get_disabled (MX_WIDGET (actor)))
+  if (mx_widget_get_disabled (MX_WIDGET (toggle)))
     return FALSE;
 
 
@@ -503,6 +512,8 @@ mx_toggle_init (MxToggle *self)
   self->priv->handle = g_object_new (MX_TYPE_TOGGLE_HANDLE,
                                      "reactive", TRUE, NULL);
   clutter_actor_set_parent (self->priv->handle, CLUTTER_ACTOR (self));
+  g_object_bind_property (self, "disabled", self->priv->handle, "disabled",
+                          G_BINDING_SYNC_CREATE);
 
   timeline = clutter_timeline_new (300);
   g_signal_connect (timeline, "new-frame",
@@ -548,16 +559,33 @@ mx_toggle_set_active (MxToggle *toggle,
     {
       ClutterTimeline *timeline;
 
-      timeline = clutter_alpha_get_timeline (priv->alpha);
+      priv->active = active;
 
-      if (clutter_timeline_is_playing (timeline))
-        return;
+      if (active)
+        mx_stylable_set_style_pseudo_class (MX_STYLABLE (toggle), "checked");
+      else
+        mx_stylable_set_style_pseudo_class (MX_STYLABLE (toggle), NULL);
+
+      g_object_notify (G_OBJECT (toggle), "active");
+
+
+      /* don't run an animation if the actor is not mapped */
+      if (!CLUTTER_ACTOR_IS_MAPPED (CLUTTER_ACTOR (toggle)))
+        {
+          priv->position = (active) ? 1 : 0;
+          return;
+        }
+
+      timeline = clutter_alpha_get_timeline (priv->alpha);
 
       if (active)
         clutter_timeline_set_direction (timeline, CLUTTER_TIMELINE_FORWARD);
       else
         clutter_timeline_set_direction (timeline, CLUTTER_TIMELINE_BACKWARD);
 
+      if (clutter_timeline_is_playing (timeline))
+        return;
+
       clutter_timeline_rewind (timeline);
 
       if (priv->drag_offset > -1)
@@ -571,16 +599,6 @@ mx_toggle_set_active (MxToggle *toggle,
         }
 
       clutter_timeline_start (timeline);
-
-      priv->active = active;
-
-      if (active)
-        mx_stylable_set_style_pseudo_class (MX_STYLABLE (toggle), "checked");
-      else
-        mx_stylable_set_style_pseudo_class (MX_STYLABLE (toggle), NULL);
-
-
-      g_object_notify (G_OBJECT (toggle), "active");
     }
 }
 
