@@ -1,3 +1,4 @@
+/* -*- mode: C; c-file-style: "gnu"; indent-tabs-mode: nil; -*- */
 /*
  * mx-utils.h: General utility functions used in Moblin
  *
@@ -31,6 +32,7 @@
 #define __MX_UTILS_H__
 
 #include <glib.h>
+#include <mx/mx.h>
 
 G_BEGIN_DECLS
 
@@ -38,6 +40,17 @@ void   mx_set_locale ();
 
 gchar *mx_utils_format_time (GTimeVal *time_);
 
+
+void mx_allocate_align_fill (ClutterActor    *child,
+                             ClutterActorBox *childbox,
+                             MxAlign          x_alignment,
+                             MxAlign          y_alignment,
+                             gboolean         x_fill,
+                             gboolean         y_fill);
+
+void mx_actor_box_clamp_to_pixels (ClutterActorBox *box);
+
+
 G_END_DECLS
 
 #endif /* __MX_UTILS_H__ */
