@@ -22,6 +22,19 @@
  *
  */
 
+/**
+ * SECTION:mx-window
+ * @short_description: an object that represents a platform-specific window
+ *
+ * #MxWindow is a platform-specific window, providing functions for moving,
+ * resizing, icons and rotation. Every window has an associated #ClutterStage
+ * in which its children are kept. This #ClutterStage can be used in the
+ * normal way, but to take advantage of automatic resizing and rotation, the
+ * #MxWindow functions should be used.
+ *
+ * When the #MxWindow loses its last reference, its contents are destroyed.
+ */
+
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
@@ -57,6 +70,7 @@ struct _MxWindowPrivate
   ClutterActor *toolbar;
   ClutterActor *child;
   ClutterActor *resize_grip;
+  ClutterActor *debug_actor;
 
   MxWindowRotation  rotation;
   ClutterTimeline  *rotation_timeline;
@@ -78,6 +92,7 @@ enum
   PROP_TOOLBAR,
   PROP_SMALL_SCREEN,
   PROP_FULLSCREEN,
+  PROP_TITLE,
   PROP_ICON_NAME,
   PROP_ICON_COGL_TEXTURE,
   PROP_CLUTTER_STAGE,
@@ -102,7 +117,8 @@ mx_window_get_property (GObject    *object,
                         GValue     *value,
                         GParamSpec *pspec)
 {
-  MxWindowPrivate *priv = MX_WINDOW (object)->priv;
+  MxWindow *self = MX_WINDOW (object);
+  MxWindowPrivate *priv = self->priv;
 
   switch (property_id)
     {
@@ -122,6 +138,10 @@ mx_window_get_property (GObject    *object,
       g_value_set_boolean (value, priv->fullscreen);
       break;
 
+    case PROP_TITLE:
+      g_value_set_string (value, mx_window_get_title (self));
+      break;
+
     case PROP_ICON_NAME:
       g_value_set_string (value, priv->icon_name);
       break;
@@ -170,6 +190,10 @@ mx_window_set_property (GObject      *object,
                                  g_value_get_boolean (value));
       break;
 
+    case PROP_TOOLBAR:
+      mx_window_set_toolbar (window, g_value_get_object (value));
+      break;
+
     case PROP_SMALL_SCREEN:
       mx_window_set_small_screen (window,
                                   g_value_get_boolean (value));
@@ -177,6 +201,11 @@ mx_window_set_property (GObject      *object,
 
     case PROP_FULLSCREEN:
       mx_window_set_fullscreen (window, g_value_get_boolean (value));
+      break;
+
+    case PROP_TITLE:
+      mx_window_set_title (window, g_value_get_string (value));
+      break;
 
     case PROP_ICON_NAME:
       mx_window_set_icon_name (window,
@@ -208,7 +237,8 @@ mx_window_set_property (GObject      *object,
 static void
 mx_window_dispose (GObject *object)
 {
-  MxWindowPrivate *priv = MX_WINDOW (object)->priv;
+  MxWindow *self = MX_WINDOW (object);
+  MxWindowPrivate *priv = self->priv;
 
   if (priv->icon_texture)
     {
@@ -217,11 +247,7 @@ mx_window_dispose (GObject *object)
     }
 
   if (priv->toolbar)
-    {
-      g_object_remove_weak_pointer (G_OBJECT (priv->toolbar),
-                                    (gpointer *)&priv->toolbar);
-      priv->toolbar = NULL;
-    }
+    mx_window_set_toolbar (self, NULL);
 
   if (priv->resize_grip)
     {
@@ -313,6 +339,137 @@ mx_window_get_size (MxWindow *window, gfloat *width, gfloat *height)
     *height = (scale * stage_width) + ((1.f - scale) * stage_height);
 }
 
+static void
+debug_paint (ClutterActor *stage,
+             MxWindow     *window)
+{
+  MxPadding padding = { 0, };
+  static gpointer current;
+  ClutterActor *actor;
+  gfloat width, height, x, y;
+  CoglMatrix matrix;
+
+  actor = window->priv->debug_actor;
+
+  if (!actor)
+    return;
+
+  if (MX_IS_WIDGET (actor))
+    mx_widget_get_padding (MX_WIDGET (actor), &padding);
+
+
+  clutter_actor_get_size (actor, &width, &height);
+  clutter_actor_get_transformed_position (actor, &x, &y);
+
+
+    {
+      ClutterActor *cur_actor;
+      GSList *parents = NULL, *l;
+
+      cogl_get_modelview_matrix (&matrix);
+
+      /* Get the complete modelview matrix for light by applying all of
+         its parent transformations as well as its own in reverse */
+      for (cur_actor = actor;
+           cur_actor != stage;
+           cur_actor = clutter_actor_get_parent (cur_actor))
+        parents = g_slist_prepend (parents, cur_actor);
+
+      for (l = parents; l; l = l->next)
+        {
+          CoglMatrix actor_matrix;
+
+          cogl_matrix_init_identity (&actor_matrix);
+          clutter_actor_get_transformation_matrix (CLUTTER_ACTOR (l->data),
+                                                   &actor_matrix);
+
+          cogl_matrix_multiply (&matrix,
+                                &matrix,
+                                &actor_matrix);
+        }
+
+      g_slist_free (parents);
+    }
+
+  cogl_push_matrix ();
+
+  cogl_set_modelview_matrix (&matrix);
+
+
+  if (current != actor)
+    {
+      current = actor;
+
+      const char *id = clutter_actor_get_name (current);
+      const char *class = NULL;
+      const char *type_name = G_OBJECT_TYPE_NAME (current);
+
+      if (MX_IS_STYLABLE (current))
+        class = mx_stylable_get_style_class (current);
+
+      g_debug ("%s%s%s%s%s (%.1f, %.1f) (%.1f x %.1f)",
+               (type_name) ? type_name : "",
+               (class) ? "." : "",
+               (class) ? class : "",
+               (id) ? "#" : "",
+               (id) ? id : "",
+               x, y,
+               width, height);
+    }
+
+  cogl_set_source_color4f (0, 0, 1, 0.5);
+  cogl_rectangle (0, 0, width, height);
+
+  cogl_set_source_color4f (0, 1, 0, 0.5);
+  cogl_rectangle (padding.left, padding.top,
+                  width - padding.right,
+                  height - padding.bottom);
+
+  cogl_set_source_color4f (0, 0, 0, 1);
+  cogl_path_rectangle (0, 0, width, height);
+  cogl_path_stroke ();
+
+  cogl_set_source_color4f (1, 1, 1, 1);
+  cogl_path_rectangle (1, 1, width - 1, height - 1);
+  cogl_path_stroke ();
+
+  cogl_pop_matrix ();
+}
+
+static void
+debug_actor_notify (gpointer debug_actor_p,
+                    GObject *actor)
+{
+  *((gpointer *)debug_actor_p) = NULL;
+}
+
+static gboolean
+debug_captured_event (ClutterActor *actor,
+                      ClutterEvent *event,
+                      gpointer      data)
+{
+  MxWindowPrivate *priv = MX_WINDOW (data)->priv;
+
+  if (event->type == CLUTTER_MOTION)
+    {
+      ClutterMotionEvent *m_event = (ClutterMotionEvent *) event;
+
+      if (priv->debug_actor)
+        g_object_weak_unref (G_OBJECT (priv->debug_actor), debug_actor_notify,
+                             &priv->debug_actor);
+
+      priv->debug_actor = clutter_stage_get_actor_at_pos (CLUTTER_STAGE (actor),
+                                                          CLUTTER_PICK_ALL,
+                                                          m_event->x,
+                                                          m_event->y);
+      g_object_weak_ref (G_OBJECT (priv->debug_actor), debug_actor_notify,
+                         &priv->debug_actor);
+    }
+
+  return FALSE;
+}
+
+
 static void
 mx_window_post_paint_cb (ClutterActor *actor, MxWindow *window)
 {
@@ -329,6 +486,8 @@ mx_window_post_paint_cb (ClutterActor *actor, MxWindow *window)
   mx_window_get_size (window, &width, &height);
   clutter_actor_get_size (actor, &stage_width, &stage_height);
 
+  cogl_push_matrix ();
+
   /* Adjust for rotation */
   cogl_translate ((stage_width - width) / 2.f,
                   (stage_height - height) / 2.f,
@@ -346,6 +505,11 @@ mx_window_post_paint_cb (ClutterActor *actor, MxWindow *window)
 
   cogl_rectangle (0, 1, 1, height - 1);
   cogl_rectangle (width - 1, 1, width, height - 1);
+
+  cogl_pop_matrix ();
+
+  if (_mx_debug (MX_DEBUG_LAYOUT))
+    debug_paint (actor, window);
 }
 
 static void
@@ -484,6 +648,14 @@ mx_window_fullscreen_set_cb (ClutterStage *stage,
   clutter_actor_queue_relayout (CLUTTER_ACTOR (stage));
 }
 
+static void
+mx_window_title_cb (ClutterStage *stage,
+                    GParamSpec   *pspec,
+                    MxWindow     *self)
+{
+  g_object_notify (G_OBJECT (self), "title");
+}
+
 static void
 mx_window_destroy_cb (ClutterStage *stage, MxWindow *self)
 {
@@ -559,11 +731,8 @@ mx_window_constructed (GObject *object)
                              (gpointer *)&priv->stage);
   g_object_set_qdata (G_OBJECT (priv->stage), window_quark, object);
 
-  priv->has_toolbar = TRUE;
-  priv->toolbar = mx_toolbar_new ();
-  clutter_container_add_actor (CLUTTER_CONTAINER (priv->stage), priv->toolbar);
-  g_object_add_weak_pointer (G_OBJECT (priv->toolbar),
-                             (gpointer *)&priv->toolbar);
+  if (!priv->toolbar && priv->has_toolbar)
+    mx_window_set_toolbar (self, MX_TOOLBAR (mx_toolbar_new ()));
 
   priv->resize_grip = mx_icon_new ();
   mx_stylable_set_style_class (MX_STYLABLE (priv->resize_grip), "ResizeGrip");
@@ -580,10 +749,10 @@ mx_window_constructed (GObject *object)
                           G_CALLBACK (mx_window_post_paint_cb), object);
   g_signal_connect (priv->stage, "allocation-changed",
                     G_CALLBACK (mx_window_allocation_changed_cb), object);
-  g_signal_connect (priv->toolbar, "allocation-changed",
-                    G_CALLBACK (mx_window_allocation_changed_cb), object);
   g_signal_connect (priv->stage, "notify::fullscreen-set",
                     G_CALLBACK (mx_window_fullscreen_set_cb), object);
+  g_signal_connect (priv->stage, "notify::title",
+                    G_CALLBACK (mx_window_title_cb), object);
   g_signal_connect (priv->stage, "destroy",
                     G_CALLBACK (mx_window_destroy_cb), object);
   g_signal_connect (priv->stage, "actor-added",
@@ -593,6 +762,10 @@ mx_window_constructed (GObject *object)
   g_signal_connect (priv->stage, "notify::user-resizable",
                     G_CALLBACK (mx_window_user_resizable_cb), object);
 
+  if (_mx_debug (MX_DEBUG_LAYOUT))
+    g_signal_connect (priv->stage, "captured-event",
+                      G_CALLBACK (debug_captured_event), object);
+
   g_object_set (G_OBJECT (priv->stage), "use-alpha", TRUE, NULL);
 
 #ifdef HAVE_X11
@@ -626,7 +799,7 @@ mx_window_class_init (MxWindowClass *klass)
                                "Toolbar",
                                "The MxToolbar associated with the window.",
                                MX_TYPE_TOOLBAR,
-                               MX_PARAM_READABLE);
+                               MX_PARAM_READWRITE);
   g_object_class_install_property (object_class, PROP_TOOLBAR, pspec);
 
   pspec = g_param_spec_boolean ("small-screen",
@@ -645,6 +818,13 @@ mx_window_class_init (MxWindowClass *klass)
                                 MX_PARAM_READWRITE);
   g_object_class_install_property (object_class, PROP_FULLSCREEN, pspec);
 
+  pspec = g_param_spec_string ("title",
+                               "Title",
+                               "Title to use for the window.",
+                               NULL,
+                               MX_PARAM_READWRITE);
+  g_object_class_install_property (object_class, PROP_TITLE, pspec);
+
   pspec = g_param_spec_string ("icon-name",
                                "Icon name",
                                "Icon name to use for the window icon.",
@@ -756,6 +936,7 @@ mx_window_init (MxWindow *self)
   priv->rotation_timeline = clutter_timeline_new (400);
   priv->rotation_alpha = clutter_alpha_new_full (priv->rotation_timeline,
                                                  CLUTTER_EASE_IN_OUT_QUAD);
+  priv->has_toolbar = TRUE;
 
   g_signal_connect (priv->rotation_timeline, "new-frame",
                     G_CALLBACK (mx_window_rotation_new_frame_cb), self);
@@ -820,7 +1001,7 @@ mx_window_new_with_clutter_stage (ClutterStage *stage)
  *
  * Gets the #MxWindow parent of the #ClutterStage, if it exists.
  *
- * Returns: A #MxWindow, or %NULL
+ * Returns: (transfer none): A #MxWindow, or %NULL
  */
 MxWindow *
 mx_window_get_for_stage (ClutterStage *stage)
@@ -875,7 +1056,7 @@ mx_window_set_child (MxWindow     *window,
  *
  * Get the primary child of the window. See mx_window_set_child().
  *
- * Returns: A #ClutterActor, or %NULL
+ * Returns: (transfer none): A #ClutterActor, or %NULL
  */
 ClutterActor*
 mx_window_get_child (MxWindow *window)
@@ -947,7 +1128,7 @@ mx_window_get_has_toolbar (MxWindow *window)
  *
  * Retrieves the toolbar associated with the window.
  *
- * Returns: A #MxToolbar
+ * Returns: (transfer none): A #MxToolbar
  */
 MxToolbar *
 mx_window_get_toolbar (MxWindow *window)
@@ -956,6 +1137,56 @@ mx_window_get_toolbar (MxWindow *window)
   return MX_TOOLBAR (window->priv->toolbar);
 }
 
+/**
+ * mx_window_set_toolbar:
+ * @window: (allow-none): A #MxWindow
+ *
+ * Sets the toolbar associated with the window.
+ *
+ * Since: 1.2
+ */
+void
+mx_window_set_toolbar (MxWindow  *window,
+                       MxToolbar *toolbar)
+{
+  MxWindowPrivate *priv;
+
+  g_return_if_fail (MX_IS_WINDOW (window));
+  g_return_if_fail (!toolbar || MX_IS_TOOLBAR (toolbar));
+
+  priv = window->priv;
+
+  if (priv->toolbar == (ClutterActor *)toolbar)
+    return;
+
+  /* Remove old toolbar */
+  if (priv->toolbar)
+    {
+      g_signal_handlers_disconnect_by_func (priv->toolbar,
+                                            mx_window_allocation_changed_cb,
+                                            window);
+      g_object_remove_weak_pointer (G_OBJECT (priv->toolbar),
+                                    (gpointer *)&priv->toolbar);
+      clutter_container_remove_actor (CLUTTER_CONTAINER (priv->stage),
+                                      priv->toolbar);
+    }
+
+  priv->toolbar = (ClutterActor *)toolbar;
+
+  /* Add new toolbar */
+  if (toolbar)
+    {
+      clutter_container_add_actor (CLUTTER_CONTAINER (priv->stage),
+                                   priv->toolbar);
+      g_object_add_weak_pointer (G_OBJECT (priv->toolbar),
+                                 (gpointer *)&priv->toolbar);
+      g_signal_connect (priv->toolbar, "allocation-changed",
+                        G_CALLBACK (mx_window_allocation_changed_cb), window);
+    }
+
+  priv->has_toolbar = priv->toolbar ? TRUE : FALSE;
+}
+
 /**
  * mx_window_get_small_screen:
  * @window: A #MxWindow
@@ -1005,6 +1236,8 @@ mx_window_set_small_screen (MxWindow *window, gboolean small_screen)
  *
  * Returns: %TRUE if the window has been set to be in fullscreen mode,
  *   otherwise %FALSE
+ *
+ * Since: 1.2
  */
 gboolean
 mx_window_get_fullscreen (MxWindow *window)
@@ -1025,6 +1258,8 @@ mx_window_get_fullscreen (MxWindow *window)
  * fullscreen. Setting this property is only a request to the underlying
  * window system.
  * </para></note>
+ *
+ * Since: 1.2
  */
 void
 mx_window_set_fullscreen (MxWindow *window,
@@ -1043,6 +1278,43 @@ mx_window_set_fullscreen (MxWindow *window,
     }
 }
 
+/**
+ * mx_window_set_title:
+ * @window: A #MxWindow
+ * @title: A string to use for the window title name
+ *
+ * Sets the title used for the window, the results of which are
+ * window-system specific.
+ *
+ * Since: 1.2
+ */
+void
+mx_window_set_title (MxWindow    *window,
+                     const gchar *title)
+{
+  g_return_if_fail (MX_IS_WINDOW (window));
+  g_return_if_fail (title != NULL);
+
+  clutter_stage_set_title (CLUTTER_STAGE (window->priv->stage), title);
+}
+
+/**
+ * mx_window_get_title:
+ * @window: A #MxWindow
+ *
+ * Retrieves the title used for the window.
+ *
+ * Returns: The title used for the window
+ *
+ * Since: 1.2
+ */
+const gchar *
+mx_window_get_title (MxWindow *window)
+{
+  g_return_val_if_fail (MX_IS_WINDOW (window), NULL);
+  return clutter_stage_get_title (CLUTTER_STAGE (window->priv->stage));
+}
+
 /**
  * mx_window_set_icon_name:
  * @window: A #MxWindow
@@ -1131,7 +1403,7 @@ mx_window_set_icon_from_cogl_texture (MxWindow   *window,
  *
  * Gets the #ClutterStage managed by the window.
  *
- * Returns: A #ClutterStage
+ * Returns: (transfer none): A #ClutterStage
  */
 ClutterStage *
 mx_window_get_clutter_stage (MxWindow *window)
@@ -1200,6 +1472,8 @@ mx_window_set_window_position (MxWindow *window,
  * Retrieves the size of the display area of the window, taking into
  * account any window border. This includes the area occupied by the
  * window's toolbar, if it's enabled.
+ *
+ * Since: 1.2
  */
 void
 mx_window_get_window_size (MxWindow *window,
@@ -1243,6 +1517,8 @@ mx_window_get_window_size (MxWindow *window,
  * Setting the window size may involve a request to the underlying windowing
  * system, and may not immediately be reflected.
  * </para></note>
+ *
+ * Since: 1.2
  */
 void
 mx_window_set_window_size (MxWindow *window,
@@ -1269,6 +1545,8 @@ mx_window_set_window_size (MxWindow *window,
  * @window: A #MxWindow
  *
  * Present the window. The actual behaviour is specific to the window system.
+ *
+ * Since: 1.2
  */
 void
 mx_window_present (MxWindow *window)
@@ -1288,6 +1566,8 @@ mx_window_present (MxWindow *window)
  * @rotation: The #MxWindowRotation
  *
  * Set the rotation of the window.
+ *
+ * Since: 1.2
  */
 void
 mx_window_set_window_rotation (MxWindow         *window,
@@ -1352,6 +1632,8 @@ mx_window_set_window_rotation (MxWindow         *window,
  * Retrieve the rotation of the window.
  *
  * Returns: An #MxWindowRotation
+ *
+ * Since: 1.2
  */
 MxWindowRotation
 mx_window_get_window_rotation (MxWindow *window)
@@ -1365,6 +1647,8 @@ mx_window_get_window_rotation (MxWindow *window)
  * @window: A #MxWindow
  *
  * Show the window
+ *
+ * Since: 1.2
  */
 void
 mx_window_show (MxWindow *window)
@@ -1380,6 +1664,7 @@ mx_window_show (MxWindow *window)
  *
  * Hide the window
  *
+ * Since: 1.2
  */
 void
 mx_window_hide (MxWindow *window)
