@@ -78,6 +78,17 @@ enum
   PROP_WINDOW
 };
 
+/* The data format expected by the _NET_WM_ICON property is defined so
+   that the alpha component is in the most significant byte and the
+   blue component is in the least significant byte which means that on
+   a big endian architecture the data format retrieved from Cogl would
+   need to be in the reverse order. */
+#if G_BYTE_ORDER == G_LITTLE_ENDIAN
+#define MX_WINDOW_X11_GET_TEXTURE_DATA_FORMAT COGL_PIXEL_FORMAT_BGRA_8888
+#else
+#define MX_WINDOW_X11_GET_TEXTURE_DATA_FORMAT COGL_PIXEL_FORMAT_ARGB_8888
+#endif
+
 static void
 mx_window_x11_get_property (GObject    *object,
                             guint       property_id,
@@ -277,7 +288,7 @@ mx_window_x11_set_wm_hints (MxWindowX11 *self)
   if ((icon_name || texture) && net_wm_icon)
     {
       guint width, height;
-      guchar *data;
+      gulong *data;
       gint size;
 
       /* Lookup icon for program name if there's no texture set */
@@ -300,10 +311,10 @@ mx_window_x11_set_wm_hints (MxWindowX11 *self)
       width = cogl_texture_get_width (texture);
       height = cogl_texture_get_height (texture);
       size = cogl_texture_get_data (texture,
-                                    COGL_PIXEL_FORMAT_BGRA_8888,
+                                    MX_WINDOW_X11_GET_TEXTURE_DATA_FORMAT,
                                     width * 4,
                                     NULL);
-      if (!size)
+      if (size != width * height * 4)
         {
           g_warning ("Unable to get texture data in "
                      "correct format for window icon");
@@ -311,19 +322,35 @@ mx_window_x11_set_wm_hints (MxWindowX11 *self)
           return;
         }
 
-      data = g_malloc (size + (sizeof (gulong) * 2));
-      ((gulong *)data)[0] = width;
-      ((gulong *)data)[1] = height;
+      data = g_new (gulong, width * height + 2);
+      data[0] = width;
+      data[1] = height;
 
       /* Get the window icon */
       if (cogl_texture_get_data (texture,
-                                 COGL_PIXEL_FORMAT_BGRA_8888,
+                                 MX_WINDOW_X11_GET_TEXTURE_DATA_FORMAT,
                                  width * 4,
-                                 data + (sizeof (gulong) * 2)) == size)
+                                 (guint8 *) (data + 2)) == size)
         {
+          /* For some inexplicable reason XChangeProperty always takes
+           * an array of longs when the format == 32 even on 64-bit
+           * architectures where sizeof(long) != 32. Therefore we need
+           * to pointlessly pad each 32-bit value with an extra 4
+           * bytes so that libX11 can remove them again to send the
+           * request. We can do this in-place if we start from the
+           * end */
+          if (sizeof (gulong) != 4)
+            {
+              const guint32 *src = (guint32 *) (data + 2) + width * height;
+              gulong *dst = data + 2 + width * height;
+
+              while (dst > data + 2)
+                *(--dst) = *(--src);
+            }
+
           /* Set the property */
           XChangeProperty (dpy, win, net_wm_icon, XA_CARDINAL,
-                           32, PropModeReplace, data,
+                           32, PropModeReplace, (unsigned char *) data,
                            (width * height) + 2);
         }
       else
@@ -432,9 +459,9 @@ mx_window_x11_allocation_changed_cb (ClutterActor           *actor,
 }
 
 static gboolean
-mx_window_x11_button_press_event_cb (ClutterActor       *actor,
-                                     ClutterButtonEvent *event,
-                                     MxWindowX11        *self)
+mx_window_x11_button_press_event_cb (ClutterActor *actor,
+                                     ClutterEvent *event,
+                                     MxWindowX11  *self)
 {
   unsigned int width, height, border_width, depth, mask;
   Window win, root, child;
@@ -453,10 +480,10 @@ mx_window_x11_button_press_event_cb (ClutterActor       *actor,
     return FALSE;
 
   /* We only care about the first mouse button */
-  if (event->button != 1)
+  if (clutter_event_get_button (event) != 1)
     return FALSE;
 
-  priv->is_moving = clutter_input_device_get_device_id (event->device);
+  priv->is_moving = clutter_event_get_device_id (event);
 
   win = clutter_x11_get_stage_window (CLUTTER_STAGE (actor));
   dpy = clutter_x11_get_default_display ();
@@ -504,14 +531,14 @@ mx_window_x11_button_release (MxWindowX11 *self)
 }
 
 static gboolean
-mx_window_x11_button_release_event_cb (ClutterActor       *actor,
-                                       ClutterButtonEvent *event,
-                                       MxWindowX11        *self)
+mx_window_x11_button_release_event_cb (ClutterActor *actor,
+                                       ClutterEvent *event,
+                                       MxWindowX11  *self)
 {
   MxWindowX11Private *priv = self->priv;
 
-  if ((clutter_input_device_get_device_id (event->device) == priv->is_moving) &&
-      (event->button == 1))
+  if (clutter_event_get_device_id (event) == priv->is_moving &&
+      clutter_event_get_button (event) == 1)
     {
       mx_window_x11_button_release (self);
       return TRUE;
@@ -528,7 +555,7 @@ mx_window_x11_captured_event_cb (ClutterActor *actor,
   MxWindowX11Private *priv = self->priv;
   ClutterActor *resize_grip = _mx_window_get_resize_grip (priv->window);
 
-  switch (event->type)
+  switch (clutter_event_type (event))
     {
     case CLUTTER_MOTION:
       /* Check if we're over the resize handle */
@@ -536,13 +563,12 @@ mx_window_x11_captured_event_cb (ClutterActor *actor,
           clutter_stage_get_user_resizable (CLUTTER_STAGE (actor)) &&
           resize_grip)
         {
-          gint x, y;
+          gfloat x, y;
           Window win;
           Display *dpy;
           gfloat height, width, rwidth, rheight;
 
           static Cursor csoutheast = 0;
-          ClutterMotionEvent *mevent = &event->motion;
 
           win = clutter_x11_get_stage_window (CLUTTER_STAGE (actor));
           dpy = clutter_x11_get_default_display ();
@@ -552,8 +578,7 @@ mx_window_x11_captured_event_cb (ClutterActor *actor,
 
           clutter_actor_get_size (actor, &width, &height);
 
-          x = mevent->x;
-          y = mevent->y;
+          clutter_event_get_coords (event, &x, &y);
 
           /* Create the resize cursor */
           if (!csoutheast)
@@ -588,9 +613,7 @@ mx_window_x11_captured_event_cb (ClutterActor *actor,
        * underneath the resize-handle.
        */
       if (priv->is_resizing)
-        return mx_window_x11_button_press_event_cb (actor,
-                                                    &event->button,
-                                                    self);
+        return mx_window_x11_button_press_event_cb (actor, event, self);
       else
         return FALSE;
 
@@ -600,9 +623,9 @@ mx_window_x11_captured_event_cb (ClutterActor *actor,
 }
 
 static gboolean
-mx_window_x11_motion_event_cb (ClutterActor       *actor,
-                               ClutterMotionEvent *event,
-                               MxWindowX11        *self)
+mx_window_x11_motion_event_cb (ClutterActor *actor,
+                               ClutterEvent *event,
+                               MxWindowX11  *self)
 {
   gint offsetx, offsety;
   gint x, y, winx, winy;
@@ -610,6 +633,7 @@ mx_window_x11_motion_event_cb (ClutterActor       *actor,
   MxWindowX11Private *priv;
   Window win, root_win, root, child;
   Display *dpy;
+  gfloat event_x, event_y;
   gfloat height, width;
 
   priv = self->priv;
@@ -618,14 +642,14 @@ mx_window_x11_motion_event_cb (ClutterActor       *actor,
    * our grabbed device.
    */
   if (!mx_window_x11_get_has_border (self) ||
-      (clutter_input_device_get_device_id (event->device) != priv->is_moving))
+      (clutter_event_get_device_id (event) != priv->is_moving))
     return FALSE;
 
   /* Check if the mouse button is still down - if the user releases the
    * mouse button while outside of the stage (which can happen), we don't
    * get the release event.
    */
-  if (!(event->modifier_state & CLUTTER_BUTTON1_MASK))
+  if (!(clutter_event_get_state (event) & CLUTTER_BUTTON1_MASK))
     {
       mx_window_x11_button_release (self);
       return TRUE;
@@ -639,8 +663,9 @@ mx_window_x11_motion_event_cb (ClutterActor       *actor,
 
   clutter_actor_get_size (actor, &width, &height);
 
-  x = event->x;
-  y = event->y;
+  clutter_event_get_coords (event, &event_x, &event_y);
+  x = (int) event_x;
+  y = (int) event_y;
 
   /* Move/resize the window if we're dragging */
   offsetx = priv->drag_x_start;
