@@ -63,7 +63,8 @@ main (int argc, char *argv[])
   ClutterActor *stage, *texture, *frame;
   GError *err = NULL;
   
-  clutter_init (&argc, &argv);
+  if (clutter_init (&argc, &argv) != CLUTTER_INIT_SUCCESS)
+    return 1;
 
   stage = clutter_stage_get_default ();
   clutter_actor_set_size (stage, 500, 300);
